#!/bin/perl5.8

=head1 Name

Dredu::Output --- Subroutines for output after data reduction

	Created: 2007-09-05
	Updated: 2007-12-30 Use of %missprint
	Updated: 2008-04-27 Subroutine &class_spread() defined. Some missspellings are corrected.
	Updated: 2011-11-27 Subroutine &class_fix(), &each_class_fix(), &store2()

=head1 Usage

use Dredu::Output

=cut

package Dredu::Output;

use Dredu;

use Exporter ();
@ISA = qw(Exporter);
@EXPORT = qw( %missprint convert_missing_to_null convert_missing_to_dot convert_missing_to_alphabet output frequencies missconv freq class_spread class_freqlist recode each_class_fix class_fix class_bool store2 );
@EXPORT_OK = qw( );


# ************************************
#  Declare Constants and Global variables
# ************************************

=head1 Global Variables

=cut

my $_count_printed = ();

%missprint = ();

# ************************************
# Methods and Subroutines
# ************************************

=head1 Exported Subroutines

=head2 convert_missing_to_null() - Convert all missing values to ''

=cut

sub convert_missing_to_null {
	foreach(values %missing_value){
		$missprint{$_} = '';
	}
}

=head2 convert_missing_to_dot() - Convert all missing values to '.'

=cut

sub convert_missing_to_dot {
	foreach(values %missing_value){
		$missprint{$_} = '.';
	}
}

=head2 convert_missing_to_alphabet() - Convert missing values to A, B, C, ...

=cut

sub convert_missing_to_alphabet {
	my $alph = 'A';
	foreach(sort numerically values %missing_value){
		$missprint{$_} = $alph++;
	}
}

=head2 output(%r) - print output data

	Argument: hash ( name => value).
		Names will be printed for the first line of output 

	To output to file, redirect or use select().

=cut

sub output {
	my ($i, @_output, @o );

		# For the first line of the output file
	unless( $_count_printed ) {
		for( $i=0; $i<=$#_ ; $i+=2 ){
			push @_output, $_[$i];
		}
		print @_output;
	}

	for( $i=1; $i<=$#_ ; $i+=2 ){
		my $v = $_[$i];
		$v = $missprint{$v}  if defined $missprint{$v};
		push @o, $v;
	}
	print @o;
	++$_count_printed;
}

########################################################
#	PWvo͗pTu[`
########################################################


sub numerically {
	if( $a =~ /[^_\d\.\-]/ ){
		return -1  unless( $b =~ /[^_\d\.\-]/ );
		return $a cmp $b;
	}
	if( $b =~ /[^_\d\.\-]/ ){
		return 1  unless( $a =~ /[^_\d\.\-]/ );
		return $a cmp $b;
	}
	$a<=>$b;
}

=head2 missconv(%r) - convert missing values

	Argument: hash ( value => count ).
		value will be converted to such as "NA" if it is a missing value

=cut

sub missconv{
	my(%v) = @_;
	foreach( keys %v ){
			# copy the count value
			# hash %missing_type and function miss() should be imported from Dredu
		$v{ $missing_type{$_} } = delete $v{$_} if( miss($_) ) ; 
	}
	return %v;
}

=head2 freq(%r) - print frequency table

	Argument: hash ( value => count ).
		value will be converted to such as "NA" if it is a missing value

=cut

sub freq {
	my(%v) = @_;
	print "\n" ;
	foreach( sort numerically keys %v ){ print "$_\t$v{$_}" ; }
	print "\n" ;
}

=head2 freqencies($f,%r) - print frequency and percent

	$f: hash ( value => count ).
	%r: hash ( value => count ).
		value will be converted to such as "NA" if it is a missing value

=cut

sub frequencies{
	my($f, %v) = @_;
	my($total, $cum) = ( 0,0 ) ;
	my(%missval, %misslabel ) ;
	foreach( keys %v ){ 
		if( $_!~/\d/ ){ $misslabel{$_}++ ; } 
		elsif( miss($_) ){ $missval{$_}++ ; }
		else { $total+=$v{$_} ; }
	}
	print "";
	foreach( sort numerically keys %v ){
		if( $misslabel{$_} ) { frprint( $f, $_, $v{$_} ) ; }
		elsif($missval{$_} ) { frprint( $f, $missing_type{$_}, $v{$_} ) ; }
		elsif($total>0) { frprint( $f, $_, $v{$_}, 100*$v{$_}/$total, $cum+=$v{$_} , 100*$cum/$total ) ; }
	}
	frprint( $f, "Total", $cum, 100*$cum/$total, "\n" ) if $total>0;
}

sub per{
	sprintf( "%5.1f", 100*$_[0] );
}

=head2 class_spread($r) - print frequencies etc. in a spreadsheet form

	$r: Refrence to hashes ( pair of values X, Y => count ).
		Missing values of Y is exempted from counting frequency.
		Beginning of each line:
			#: Frequencies
			%: Valid parcentages
			C#: Cumulative frequencies
			C%: Cumulative valid parcentages
			S: Number of survived cases
			H: Hazard
=cut


sub class_spread {
	my $r = shift;
	foreach( sort keys %$r ) {
		printf "\n$_\n" ;
		each_class_spread( %{$$r{$_}} );
	}
}

=head2 each_class_spread(%r) - print frequencies etc. in a spreadsheet form

	%r: hash ( pair of values X, Y => count ).
		Missing values of Y is exempted from counting frequency.
		Beginning of each line:
			#: Frequencies
			%: Valid parcentages
			C#: Cumulative frequencies
			C%: Cumulative valid parcentages
			S: Number of survived cases
			H: Hazard
=cut


sub each_class_spread {
	my(%v) = @_;	# $v{x,y}
	my %m;     	# $n{X}{Y} = $v{X,Y} for missing Y
	my %n;     	# $n{X}{Y} = $v{X,Y} for valid Y
	my @x;     	# Array of X values (including missing values)
	my %y;     	# $y{X}[1,2,3,...]
	my %total; 	# %total{1} = Sum of valid cases where X=1
	
	foreach( keys %v ) {
		my( $a , $b ) = split($; , $_) ;
		if( miss($b) ){
			$m{$a}{$b} = $v{$_} ;
			next;
		}
		$n{$a}{$b} = $v{$_} ;
		$total{$a} +=$v{$_} ;
	}
	@x =  sort numerically keys %total;

	foreach(@x){
		my @b = sort numerically keys %{$n{$_}};
		$y{$_} = \@b;
	}

	my ($a, $b );
	foreach $a (@x) {
		my ( @freq, @pct, @cum, @cumpct, @survive, @hazard );
		my $cum = 0;
		foreach $b ( @{$y{$a}} ) {
			push @freq, $n{$a}{$b};
			push @pct, 100 * $n{$a}{$b} / $total{$a}  if $total{$a};
			$cum += $n{$a}{$b};
			push @cum, $cum;
			push @cumpct, 100 * $cum / $total{$a}  if $total{$a};
			push @survive, $total{$a}-$cum;
			my $p = $total{$a}-$cum+$n{$a}{$b};
			push @hazard, 100 * $n{$a}{$b} / $p  if $p;
		}
		print 'Y' ,'X', @{ $y{$a} };
		print '#' , $a, @freq;
		print '%', $a, @pct;
		print 'C#' , $a, @cum;
		print 'C%' , $a, @cumpct;
		print 'S', $a, @survive;
		print 'H', $a, @hazard;
		print "" ;
	}
	print 'Missing Y cases:';
	foreach $a(sort numerically keys %m){
		foreach $b(sort numerically keys %{$m{$a}}){
			printf "(%s, %s) = %d\n" , misstype($a, $b) , $m{$a}{$b} ;
		}
	}
	print "";
}

=head2 store2( $ref, $heading, $x, $y, $n ) - store frequencies to hashes

	$ref: Refrence to hashes ( pair of values X, Y => count )
	$heading: Heading to be printed (as a key for each hash)
	$x: X value
	$y: Y value
	$n: Count to be added to the stored frequency

=cut

sub store2 {
	my( $ref, $heading, $x, $y, $n ) = @_ ;
	$$ref{$heading}{$x,$y} += $n ;
}

=head2 class_fix($r) - print frequencies in a fixed spreadsheet form

	$r: Refrence to hashes ( pair of values X, Y => count ).
		Missing values of Y is exempted from counting frequency.
		Beginning of each line:
			#: Frequencies
=cut

sub class_fix {
	my $r = shift;
	foreach( sort keys %$r ) {
		printf "\n$_\n" ;
		each_class_fix( %{$$r{$_}} );
	}
}

=head2 each_class_fix(%r) - print frequencies in a fixed spreadsheet form

	%r: hash ( pair of values X, Y => count ).
		Missing values of Y is exempted from counting frequency.
		Beginning of each line:
			#: Frequencies
=cut


sub each_class_fix {
	my(%v) = @_;	# $v{x,y}
	my %m;     	# $n{X}{Y} = $v{X,Y} for missing Y
	my %n;     	# $n{X}{Y} = $v{X,Y} for valid Y
	my @x;     	# Array of X values (including missing values)
	my @yvalue; 	# Array of Y values (excluding missing values)
	my %yvalue; 	# Hash to count Y values (excluding missing values)
	my %y;     	# $y{X}[1,2,3,...]
	my %total; 	# %total{1} = Sum of valid cases where X=1
	
	foreach( keys %v ) {
		my( $a , $b ) = split($; , $_) ;
		if( miss($b) ){
			$m{$a}{$b} = $v{$_} ;
			next;
		}
		$n{$a}{$b} = $v{$_} ;
		$total{$a} +=$v{$_} ;
		++$yvalue{$b} ;
	}
	@x =  sort numerically keys %total;
	@yvalue = sort numerically keys %yvalue;

	foreach(@x){
#		my @b = sort numerically keys %{$n{$_}};
		$y{$_} = \@yvalue;
	}

	print 'Y' ,'X', @yvalue;
	my ($a, $b );
	foreach $a (@x) {
		my ( @freq, @pct, @cum, @cumpct, @survive, @hazard );
		my $cum = 0;
		foreach $b ( @{$y{$a}} ) {
			push @freq, $n{$a}{$b};
			push @pct, 100 * $n{$a}{$b} / $total{$a}  if $total{$a};
			$cum += $n{$a}{$b};
			push @cum, $cum;
			push @cumpct, 100 * $cum / $total{$a}  if $total{$a};
			push @survive, $total{$a}-$cum;
			my $p = $total{$a}-$cum+$n{$a}{$b};
			push @hazard, 100 * $n{$a}{$b} / $p  if $p;
		}
#		print 'Y' ,'X', @{ $y{$a} };
		print '#' , $a, @freq;
next;
		print '%', $a, @pct;
		print 'C#' , $a, @cum;
		print 'C%' , $a, @cumpct;
		print 'S', $a, @survive;
		print 'H', $a, @hazard;
		print "" ;
	}
return;

	print 'Missing Y cases:';
	foreach $a(sort numerically keys %m){
		foreach $b(sort numerically keys %{$m{$a}}){
			printf "(%s, %s) = %d\n" , misstype($a, $b) , $m{$a}{$b} ;
		}
	}
	print "";
}

=head2 class_bool(%r) - print percent and total number for a Boolean value

	%r: hash ( pair of values X, Y => count ).
		Missing values of Y is exempted from counting frequency.
		Beginning of each line:
			%: Percent (Total number)
=cut


sub class_bool {
	my(%v) = @_;	# $v{x,y}
	my %m;     	# $n{X}{Y} = $v{X,Y} for missing Y
	my %n;     	# $n{X}{Y} = $v{X,Y} for valid Y
	my @x;     	# Array of X values (including missing values)
	my %y;     	# $y{X}[1,2,3,...]
	my %total; 	# %total{1} = Sum of valid cases where X=1
	
	foreach( keys %v ) {
		my( $a , $b ) = split($; , $_) ;
		if( miss($b) ){
			$m{$a}{$b} = $v{$_} ;
			next;
		}
		$b = ($b) ? 1 : 0 ;
		$n{$a}{$b} = $v{$_} ;
		$total{$a} +=$v{$_} ;
	}
	@x =  sort numerically keys %total;

	print 'Y' ,'X', '%', 'n', 'Total';
	my ($a, $b );
	foreach $a (@x) {
		my $total = $total{$a};
		my $freq = $n{$a}{1} ;
		my $pct = sprintf( '%5.1f', 100 * $freq / $total )  if $total;
		print '%' , $a, $pct, $freq, $total;
	}

	print 'Missing Y cases:';
	foreach $a(sort numerically keys %m){
		foreach $b(sort numerically keys %{$m{$a}}){
			printf "(%s, %s) = %d\n" , misstype($a, $b) , $m{$a}{$b} ;
		}
	}
	print "";
}

sub class_freqlist {
	my($f, %v) = @_;
	my($total, $cum) = ( 0,0 ) ;
	my(%missval, %misslabel) ;
	my($x,$y,@x,@y,%x,%y) ;
	foreach( keys %v ){
		($x,$y) = split($;) ;
		++$x{$x};
		++$y{$y};
		if( $y!~/\d/ ){ $misslabel{$y}++ ; } 
		elsif( miss($y) ){ $missval{$y}++ ; }
	}
	foreach( sort numerically  keys %x ){ push( @x, $_ ); }
	foreach( sort numerically  keys %y ){ push( @y, $_ ); }
	foreach $x ( @x ){
		$total = $cum = 0 ;
		printf( "\nX = %s\n", miss($x) ? $missing_type{$x} : $x );
		foreach( @y ){
			$total+=$v{$x,$_} if( exists $v{$x,$_} && ! $misslabel{$_} && ! $missval{$_} ) ;
		}
		foreach( @y ){
			next unless exists $v{$x,$_} ;
			if( $misslabel{$_} ) { &frprint( $f, $_, $v{$x,$_} ) ; }
			elsif($missval{$_} ) { &frprint( $f, $missing_type{$_}, $v{$x,$_} ) ; }
			elsif($total>0) { &frprint( $f, $_, $v{$x,$_}, 100*$v{$x,$_}/$total, $cum+=$v{$x,$_}, 100*$cum/$total ); }
		}
		&frprint( $f, "Total", $cum, 100*$cum/$total, "\n" ) if $total>0;
	}
}

sub frprint {
	my($f, @list) = @_ ;
	foreach( @list ){
		if( ! /^\d/ ) { printf( "%${f}s" , $_ ) ; }
		elsif( $_ != int ) { printf( "%${f}.1f" , $_ ) ; }
		else{ printf( "%${f}d" , $_ ) ; }
	}
	printf( "\n" );
}

1;


=head1 Author

=over 4

=item B<TANAKA Sigeto> <tsigeto@nik.sal.tohoku.ac.jp>

Graduate School of Arts and Letters, Tohoku University

Kawauti 27-1, Aoba Ku, Sendai, Miyagi, 980-8576 JAPAN

WWW: http://www.sal.tohoku.ac.jp/~tsigeto/

=back

=head1 Copyright

	Copyright (c) 2007,2008 TANAKA Sigeto

You may use, duplicate, revise, or redistribute 
this script, without permission from the copyright holder.

=cut
